@@ -40,6 +40,10 @@ def main(args=None):
         raise SystemExit(retcode)
     except KeyboardInterrupt:
         raise SystemExit(2)
+    except tox.exception.MinVersionError as e:
+        r = Reporter(None)
+        r.error(e.message)
+        raise SystemExit(1)
 
 
 def show_help(config):
@@ -51,7 +55,7 @@ def show_help(config):
             "(overridable by '-e')")
     tw.line("TOX_TESTENV_PASSENV: space-separated list of extra "
             "environment variables to be passed into test command "
-            "environemnts")
+            "environments")
 
 
 def show_help_ini(config):
@@ -211,7 +215,7 @@ class Action(object):
         if sys.platform == "win32":
             ext = os.path.splitext(str(newargs[0]))[1].lower()
             if ext == '.py' and self.venv:
-                newargs = [str(self.venv.getcommandpath())] + newargs
+                newargs = [str(self.venv.envconfig.envpython)] + newargs
 
         return newargs
 
@@ -233,6 +237,12 @@ class Reporter(object):
         self._reportedlines = []
         # self.cumulated_time = 0.0
 
+    def _get_verbosity(self):
+        if self.session:
+            return self.session.config.option.verbosity
+        else:
+            return 2
+
     def logpopen(self, popen, env):
         """ log information about the action.popen() created process. """
         cmd = " ".join(map(str, popen.args))
@@ -252,16 +262,17 @@ class Reporter(object):
         # self.cumulated_time += duration
         self.verbosity2("%s finish: %s after %.2f seconds" % (
             action.venvname, action.msg, duration), bold=True)
+        delattr(action, '_starttime')
 
     def startsummary(self):
         self.tw.sep("_", "summary")
 
     def info(self, msg):
-        if self.session.config.option.verbosity >= 2:
+        if self._get_verbosity() >= 2:
             self.logline(msg)
 
     def using(self, msg):
-        if self.session.config.option.verbosity >= 1:
+        if self._get_verbosity() >= 1:
             self.logline("using %s" % (msg,), bold=True)
 
     def keyboard_interrupt(self):
@@ -297,15 +308,15 @@ class Reporter(object):
         self.tw.line("%s" % msg, **opts)
 
     def verbosity0(self, msg, **opts):
-        if self.session.config.option.verbosity >= 0:
+        if self._get_verbosity() >= 0:
             self.logline("%s" % msg, **opts)
 
     def verbosity1(self, msg, **opts):
-        if self.session.config.option.verbosity >= 1:
+        if self._get_verbosity() >= 1:
             self.logline("%s" % msg, **opts)
 
     def verbosity2(self, msg, **opts):
-        if self.session.config.option.verbosity >= 2:
+        if self._get_verbosity() >= 2:
             self.logline("%s" % msg, **opts)
 
     # def log(self, msg):
@@ -313,6 +324,8 @@ class Reporter(object):
 
 
 class Session:
+    """ (unstable API).  the session object that ties
+    together configuration, reporting, venv creation, testing. """
 
     def __init__(self, config, popen=subprocess.Popen, Report=Reporter):
         self.config = config
@@ -332,13 +345,24 @@ class Session:
             ]
         except LookupError:
             raise SystemExit(1)
+        except tox.exception.ConfigError as e:
+            self.report.error(str(e))
+            raise SystemExit(1)
         self._actions = []
 
+    @property
+    def hook(self):
+        return self.config.pluginmanager.hook
+
     def _makevenv(self, name):
         envconfig = self.config.envconfigs.get(name, None)
         if envconfig is None:
             self.report.error("unknown environment %r" % name)
             raise LookupError(name)
+        elif envconfig.envdir == self.config.toxinidir:
+            self.report.error(
+                "venv %r in %s would delete project" % (name, envconfig.envdir))
+            raise tox.exception.ConfigError('envdir must not equal toxinidir')
         venv = VirtualEnv(envconfig=envconfig, session=self)
         self._name2venv[name] = venv
         return venv
@@ -357,14 +381,6 @@ class Session:
 
     def runcommand(self):
         self.report.using("tox-%s from %s" % (tox.__version__, tox.__file__))
-        if self.config.minversion:
-            minversion = NormalizedVersion(self.config.minversion)
-            toxversion = NormalizedVersion(tox.__version__)
-            if toxversion < minversion:
-                self.report.error(
-                    "tox version is %s, required is at least %s" % (
-                        toxversion, minversion))
-                raise SystemExit(1)
         if self.config.option.showconfig:
             self.showconfig()
         elif self.config.option.listenvs:
@@ -532,12 +548,12 @@ class Session:
                 # write out version dependency information
                 action = self.newaction(venv, "envreport")
                 with action:
-                    pip = venv.getcommandpath("pip")
-                    # we can't really call internal helpers here easily :/
-                    # output = venv._pcall([str(pip), "freeze"],
-                    #                      cwd=self.config.toxinidir,
-                    #                      action=action)
-                    output = py.process.cmdexec("%s freeze" % (pip))
+                    args = venv.envconfig.list_dependencies_command
+                    output = venv._pcall(args,
+                                         cwd=self.config.toxinidir,
+                                         action=action)
+                    # the output contains a mime-header, skip it
+                    output = output.split("\n\n")[-1]
                     packages = output.strip().split("\n")
                     action.setactivity("installed", ",".join(packages))
                     envlog = self.resultlog.get_envlog(venv.name)
@@ -551,7 +567,9 @@ class Session:
         if not self.config.option.notest:
             if venv.status:
                 return
+            self.hook.tox_runtest_pre(venv=venv)
             venv.test(redirect=redirect)
+            self.hook.tox_runtest_post(venv=venv)
         else:
             venv.status = "skipped tests"
 
